# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import extensions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestExtensionsClient(base.BaseServiceTest):

    FAKE_SHOW_EXTENSION = {
        "extension": {
            "updated": "2011-06-09T00:00:00Z",
            "name": "Multinic",
            "links": [],
            "namespace":
            "http://docs.openstack.org/compute/ext/multinic/api/v1.1",
            "alias": "NMN",
            "description": '\u2740(*\xb4\u25e1`*)\u2740'
        }
    }

    def setUp(self):
        super(TestExtensionsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = extensions_client.ExtensionsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_extensions(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_extensions,
            'tempest.lib.common.rest_client.RestClient.get',
            {"extensions": []},
            bytes_body)

    def test_list_extensions_with_str_body(self):
        self._test_list_extensions()

    def test_list_extensions_with_bytes_body(self):
        self._test_list_extensions(bytes_body=True)

    def _test_show_extension(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_extension,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_EXTENSION,
            bytes_body,
            extension_alias="NMN")

    def test_show_extension_with_str_body(self):
        self._test_show_extension()

    def test_show_extension_with_bytes_body(self):
        self._test_show_extension(bytes_body=True)



# Copyright 2013 Huawei Technologies Co.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class VolumesActionsTest(base.BaseVolumeAdminTest):
    """Test volume actions"""

    create_default_network = True

    def _create_reset_and_force_delete_temp_volume(self, status=None):
        # Create volume, reset volume status, and force delete temp volume
        temp_volume = self.create_volume()
        if status:
            self.admin_volume_client.reset_volume_status(
                temp_volume['id'], status=status)
            waiters.wait_for_volume_resource_status(
                self.volumes_client, temp_volume['id'], status)
        self.admin_volume_client.force_delete_volume(temp_volume['id'])
        self.volumes_client.wait_for_resource_deletion(temp_volume['id'])

    @decorators.idempotent_id('d063f96e-a2e0-4f34-8b8a-395c42de1845')
    def test_volume_reset_status(self):
        """Test resetting volume status

        Reset volume status to available->error->available->maintenance
        """
        volume = self.create_volume()
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.volumes_client, volume['id'], 'available')
        self.addCleanup(self.admin_volume_client.reset_volume_status,
                        volume['id'], status='available')
        for status in ['error', 'available', 'maintenance']:
            self.admin_volume_client.reset_volume_status(
                volume['id'], status=status)
            waiters.wait_for_volume_resource_status(
                self.volumes_client, volume['id'], status)

    @decorators.idempotent_id('21737d5a-92f2-46d7-b009-a0cc0ee7a570')
    def test_volume_force_delete_when_volume_is_creating(self):
        """Test force deleting volume when its status is creating"""
        self._create_reset_and_force_delete_temp_volume('creating')

    @decorators.idempotent_id('db8d607a-aa2e-4beb-b51d-d4005c232011')
    def test_volume_force_delete_when_volume_is_attaching(self):
        """Test force deleting volume when its status is attaching"""
        self._create_reset_and_force_delete_temp_volume('attaching')

    @decorators.idempotent_id('3e33a8a8-afd4-4d64-a86b-c27a185c5a4a')
    def test_volume_force_delete_when_volume_is_error(self):
        """Test force deleting volume when its status is error"""
        self._create_reset_and_force_delete_temp_volume('error')

    @decorators.idempotent_id('b957cabd-1486-4e21-90cf-a9ed3c39dfb2')
    def test_volume_force_delete_when_volume_is_maintenance(self):
        """Test force deleting volume when its status is maintenance"""
        self._create_reset_and_force_delete_temp_volume('maintenance')

    @decorators.idempotent_id('d38285d9-929d-478f-96a5-00e66a115b81')
    @utils.services('compute')
    def test_force_detach_volume(self):
        """Test force detaching volume when its status is error"""
        # Create a server and a volume
        server_id = self.create_server()['id']
        volume_id = self.create_volume()['id']

        # Request Cinder to map & export volume (it's not attached to instance)
        self.volumes_client.attach_volume(
            volume_id,
            instance_uuid=server_id,
            mountpoint='/dev/%s' % CONF.compute.volume_device_name)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume_id, 'in-use')
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.volumes_client, volume_id, 'available')
        self.addCleanup(self.volumes_client.detach_volume, volume_id)
        attachment = self.volumes_client.show_volume(
            volume_id)['volume']['attachments'][0]

        # Reset volume's status to error
        self.admin_volume_client.reset_volume_status(volume_id, status='error')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume_id, 'error')

        # The force detach volume calls works because the volume is not really
        # connected to the instance (it is safe), otherwise it would be
        # rejected for security reasons (bug #2004555).
        self.admin_volume_client.force_detach_volume(
            volume_id, connector=None,
            attachment_id=attachment['attachment_id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume_id, 'available')
        vol_info = self.volumes_client.show_volume(volume_id)['volume']
        self.assertEmpty(vol_info['attachments'])



# Copyright (c) 2015 Deutsche Telekom AG
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.test_discover import plugins


class FakePlugin(plugins.TempestPlugin):
    expected_load_test = ["my/test/path", "/home/dir"]
    expected_service_clients = [{'foo': 'bar'}]

    def load_tests(self):
        return self.expected_load_test

    def register_opts(self, conf):
        return

    def get_opt_lists(self):
        return []

    def get_service_clients(self):
        return self.expected_service_clients


class FakeStevedoreObj(object):
    obj = FakePlugin()

    @property
    def name(self):
        return self._name

    def __init__(self, name='Test1'):
        self._name = name


class FakePluginNoServiceClients(plugins.TempestPlugin):

    def load_tests(self):
        return []

    def register_opts(self, conf):
        return

    def get_opt_lists(self):
        return []


class FakeStevedoreObjNoServiceClients(object):
    obj = FakePluginNoServiceClients()

    @property
    def name(self):
        return self._name

    def __init__(self, name='Test2'):
        self._name = name



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v2 import endpoints_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestEndpointsClient(base.BaseServiceTest):
    FAKE_CREATE_ENDPOINT = {
        "endpoint": {
            "id": 1,
            "tenantId": 1,
            "region": "North",
            "type": "compute",
            "publicURL": "https://compute.north.public.com/v1",
            "internalURL": "https://compute.north.internal.com/v1",
            "adminURL": "https://compute.north.internal.com/v1"
        }
    }

    FAKE_LIST_ENDPOINTS = {
        "endpoints": [
            {
                "id": 1,
                "tenantId": "1",
                "region": "North",
                "type": "compute",
                "publicURL": "https://compute.north.public.com/v1",
                "internalURL": "https://compute.north.internal.com/v1",
                "adminURL": "https://compute.north.internal.com/v1"
            },
            {
                "id": 2,
                "tenantId": "1",
                "region": "South",
                "type": "compute",
                "publicURL": "https://compute.north.public.com/v1",
                "internalURL": "https://compute.north.internal.com/v1",
                "adminURL": "https://compute.north.internal.com/v1"
            }
        ]
    }

    def setUp(self):
        super(TestEndpointsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = endpoints_client.EndpointsClient(fake_auth,
                                                       'identity', 'regionOne')

    def _test_create_endpoint(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_endpoint,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_ENDPOINT,
            bytes_body,
            service_id="b344506af7644f6794d9cb316600b020",
            region="region-demo",
            publicurl="https://compute.north.public.com/v1",
            adminurl="https://compute.north.internal.com/v1",
            internalurl="https://compute.north.internal.com/v1")

    def _test_list_endpoints(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_endpoints,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ENDPOINTS,
            bytes_body)

    def test_create_endpoint_with_str_body(self):
        self._test_create_endpoint()

    def test_create_endpoint_with_bytes_body(self):
        self._test_create_endpoint(bytes_body=True)

    def test_list_endpoints_with_str_body(self):
        self._test_list_endpoints()

    def test_list_endpoints_with_bytes_body(self):
        self._test_list_endpoints(bytes_body=True)

    def test_delete_endpoint(self):
        self.check_service_client_function(
            self.client.delete_endpoint,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            endpoint_id="b344506af7644f6794d9cb316600b020",
            status=204)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

common_security_group_default_rule_info = {
    'type': 'object',
    'properties': {
        'from_port': {'type': 'integer'},
        'id': {'type': 'integer'},
        'ip_protocol': {'type': 'string'},
        'ip_range': {
            'type': 'object',
            'properties': {
                'cidr': {'type': 'string'}
            },
            'additionalProperties': False,
            'required': ['cidr'],
        },
        'to_port': {'type': 'integer'},
    },
    'additionalProperties': False,
    'required': ['from_port', 'id', 'ip_protocol', 'ip_range', 'to_port'],
}

create_get_security_group_default_rule = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'security_group_default_rule':
                common_security_group_default_rule_info
        },
        'additionalProperties': False,
        'required': ['security_group_default_rule']
    }
}

delete_security_group_default_rule = {
    'status_code': [204]
}

list_security_group_default_rules = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'security_group_default_rules': {
                'type': 'array',
                'items': common_security_group_default_rule_info
            }
        },
        'additionalProperties': False,
        'required': ['security_group_default_rules']
    }
}



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr
import testtools

from oslo_serialization import jsonutils as json

from tempest.api.compute import base
from tempest.common import utils
from tempest.common.utils.linux import remote_client
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ServersTestJSON(base.BaseV2ComputeTest):
    """Test creating server and verifying the server attributes

    This is to create server booted from image and with disk_config 'AUTO'
    """

    disk_config = 'AUTO'
    volume_backed = False

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(ServersTestJSON, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServersTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServersTestJSON, cls).resource_setup()
        validation_resources = cls.get_class_validation_resources(
            cls.os_primary)
        cls.meta = {'hello': 'world'}
        cls.accessIPv4 = '1.1.1.1'
        cls.accessIPv6 = '0000:0000:0000:0000:0000:babe:220.12.22.2'
        cls.name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=cls.__name__ + '-server')
        cls.password = data_utils.rand_password()
        disk_config = cls.disk_config
        server_initial = cls.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='ACTIVE',
            name=cls.name,
            metadata=cls.meta,
            accessIPv4=cls.accessIPv4,
            accessIPv6=cls.accessIPv6,
            disk_config=disk_config,
            adminPass=cls.password,
            volume_backed=cls.volume_backed)
        cls.server = cls.client.show_server(server_initial['id'])['server']

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('5de47127-9977-400a-936f-abcfbec1218f')
    def test_verify_server_details(self):
        """Verify the specified server attributes are set correctly"""
        self.assertEqual(self.accessIPv4, self.server['accessIPv4'])
        # NOTE(maurosr): See http://tools.ietf.org/html/rfc5952 (section 4)
        # Here we compare directly with the canonicalized format.
        self.assertEqual(self.server['accessIPv6'],
                         str(netaddr.IPAddress(self.accessIPv6)))
        self.assertEqual(self.name, self.server['name'])
        if self.volume_backed:
            # Image is an empty string as per documentation
            self.assertEqual("", self.server['image'])
        else:
            self.assertEqual(self.image_ref, self.server['image']['id'])
        self.assert_flavor_equal(self.flavor_ref, self.server['flavor'])
        self.assertEqual(self.meta, self.server['metadata'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('9a438d88-10c6-4bcd-8b5b-5b6e25e1346f')
    def test_list_servers(self):
        """The created server should be in the list of all servers"""
        body = self.client.list_servers()
        servers = body['servers']
        found = [i for i in servers if i['id'] == self.server['id']]
        self.assertNotEmpty(found)

    @decorators.idempotent_id('585e934c-448e-43c4-acbf-d06a9b899997')
    def test_list_servers_with_detail(self):
        """The created server should be in the detailed list of all servers"""
        body = self.client.list_servers(detail=True)
        servers = body['servers']
        found = [i for i in servers if i['id'] == self.server['id']]
        self.assertNotEmpty(found)

    @decorators.idempotent_id('cbc0f52f-05aa-492b-bdc1-84b575ca294b')
    @testtools.skipUnless(CONF.validation.run_validation,
                          'Instance validation tests are disabled.')
    def test_verify_created_server_vcpus(self):
        """The created server should have the same specification as the flavor

        Verify that the number of vcpus reported by the instance matches
        the amount stated by the flavor
        """
        flavor = self.flavors_client.show_flavor(self.flavor_ref)['flavor']
        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        linux_client = remote_client.RemoteClient(
            self.get_server_ip(self.server, validation_resources),
            self.ssh_user,
            self.password,
            validation_resources['keypair']['private_key'],
            server=self.server,
            servers_client=self.client)
        output = linux_client.exec_command('grep -c ^processor /proc/cpuinfo')
        self.assertEqual(flavor['vcpus'], int(output))

    @decorators.idempotent_id('ac1ad47f-984b-4441-9274-c9079b7a0666')
    @testtools.skipUnless(CONF.validation.run_validation,
                          'Instance validation tests are disabled.')
    def test_host_name_is_same_as_server_name(self):
        """Verify the instance host name is the same as the server name"""
        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        linux_client = remote_client.RemoteClient(
            self.get_server_ip(self.server, validation_resources),
            self.ssh_user,
            self.password,
            validation_resources['keypair']['private_key'],
            server=self.server,
            servers_client=self.client)
        hostname = linux_client.exec_command("hostname").rstrip()
        msg = ('Failed while verifying servername equals hostname. Expected '
               'hostname "%s" but got "%s".' %
               (self.name, hostname.split(".")[0]))
        # NOTE(zhufl): Some images will add postfix for the hostname, e.g.,
        # if hostname is "aaa", postfix ".novalocal" may be added to it, and
        # the hostname will be "aaa.novalocal" then, so we should ignore the
        # postfix when checking whether hostname equals self.name.
        self.assertEqual(self.name.lower(), hostname.split(".")[0], msg)


class ServersTestManualDisk(ServersTestJSON):
    """Test creating server and verifying the server attributes

    This is to create server booted from image and with disk_config 'MANUAL'
    """
    disk_config = 'MANUAL'

    @classmethod
    def skip_checks(cls):
        super(ServersTestManualDisk, cls).skip_checks()
        if not CONF.compute_feature_enabled.disk_config:
            msg = "DiskConfig extension not enabled."
            raise cls.skipException(msg)


class ServersTestBootFromVolume(ServersTestJSON):
    """Test creating server and verifying the server attributes

    This is to create server booted from volume and with disk_config 'AUTO'
    """
    # Run the `ServersTestJSON` tests with a volume backed VM
    volume_backed = True

    @classmethod
    def skip_checks(cls):
        super(ServersTestBootFromVolume, cls).skip_checks()
        if not utils.get_service_list()['volume']:
            msg = "Volume service not enabled."
            raise cls.skipException(msg)


class ServersTestFqdnHostnames(base.BaseV2ComputeTest):
    """Test creating server with FQDN hostname and verifying attributes

    Starting Wallaby release, Nova sanitizes freeform characters in
    server hostname with dashes. This test verifies the same.
    """

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(ServersTestFqdnHostnames, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServersTestFqdnHostnames, cls).setup_clients()
        cls.client = cls.servers_client

    @decorators.idempotent_id('622066d2-39fc-4c09-9eeb-35903c114a0a')
    @testtools.skipUnless(
        CONF.compute_feature_enabled.hostname_fqdn_sanitization,
        'FQDN hostname sanitization is not supported.')
    @testtools.skipUnless(CONF.validation.run_validation,
                          'Instance validation tests are disabled.')
    def test_create_server_with_fqdn_name(self):
        """Test to create an instance with FQDN type name scheme"""
        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        self.server_name = 'guest-instance-1.domain.com'
        self.password = data_utils.rand_password()
        self.accessIPv4 = '2.2.2.2'
        test_server = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='ACTIVE',
            adminPass=self.password,
            name=self.server_name,
            accessIPv4=self.accessIPv4)

        """Verify the hostname within the instance is sanitized

        Freeform characters in the hostname are replaced with dashes
        """
        linux_client = remote_client.RemoteClient(
            self.get_server_ip(test_server, validation_resources),
            self.ssh_user,
            self.password,
            validation_resources['keypair']['private_key'],
            server=test_server,
            servers_client=self.client)
        hostname = linux_client.exec_command("hostname").rstrip()
        self.assertEqual('guest-instance-1-domain-com', hostname)


class ServersV294TestFqdnHostnames(base.BaseV2ComputeTest):
    """Test creating server with FQDN hostname and verifying attributes

    Starting Antelope release, Nova allows to set hostname as an FQDN
    type and allows free form characters in hostname using --hostname
    parameter with API above 2.94 .

    This is to create server with --hostname having FQDN type value having
    more than 64 characters
    """

    min_microversion = '2.94'

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(ServersV294TestFqdnHostnames, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServersV294TestFqdnHostnames, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServersV294TestFqdnHostnames, cls).resource_setup()
        cls.validation_resources = cls.get_class_validation_resources(
            cls.os_primary)
        cls.accessIPv4 = '1.1.1.1'
        cls.name = 'guest-instance-1'
        cls.password = data_utils.rand_password()
        cls.hostname = 'x' * 52 + '-guest-test.domaintest.com'
        cls.test_server = cls.create_test_server(
            validatable=True,
            validation_resources=cls.validation_resources,
            wait_until='ACTIVE',
            name=cls.name,
            accessIPv4=cls.accessIPv4,
            adminPass=cls.password,
            hostname=cls.hostname)
        cls.server = cls.client.show_server(cls.test_server['id'])['server']

    def verify_metadata_hostname(self, md_json):
        md_dict = json.loads(md_json)
        dhcp_domain = CONF.compute_feature_enabled.dhcp_domain
        if md_dict['hostname'] == f"{self.hostname}{dhcp_domain}":
            return True
        else:
            return False

    @decorators.idempotent_id('e7b05488-f9d5-4fce-91b3-e82216c52017')
    @testtools.skipUnless(CONF.validation.run_validation,
                          'Instance validation tests are disabled.')
    def test_verify_hostname_allows_fqdn(self):
        """Test to verify --hostname allows FQDN type name scheme

        Verify the hostname has FQDN value and Freeform characters
        in the hostname are allowed
        """
        self.assertEqual(
            self.hostname, self.server['OS-EXT-SRV-ATTR:hostname'])
        # Verify that metadata API has correct hostname inside guest
        linux_client = remote_client.RemoteClient(
            self.get_server_ip(self.test_server, self.validation_resources),
            self.ssh_user,
            self.password,
            self.validation_resources['keypair']['private_key'],
            server=self.test_server,
            servers_client=self.client)
        self.verify_metadata_from_api(
            self.test_server, linux_client, self.verify_metadata_hostname)



# Copyright 2017 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.volume import base
from tempest.lib import decorators


class VersionsTest(base.BaseVolumeTest):
    """Test volume versions"""

    @decorators.idempotent_id('77838fc4-b49b-4c64-9533-166762517369')
    @decorators.attr(type='smoke')
    def test_list_versions(self):
        """Test listing volume versions"""
        # NOTE: The version data is checked on service client side
        #       with JSON-Schema validation. It is enough to just call
        #       the API here.
        self.versions_client.list_versions()

    @decorators.idempotent_id('7f755ae2-caa9-4049-988c-331d8f7a579f')
    def test_show_version(self):
        """Test getting volume version details"""
        # NOTE: The version data is checked on service client side
        # with JSON-Schema validation. So we will loop through each
        # version and call show version.
        versions = self.versions_client.list_versions()['versions']
        for version_dict in versions:
            version = version_dict['id']
            major_version = version.split('.')[0]
            response = self.versions_client.show_version(major_version)
            self.assertEqual(version, response['versions'][0]['id'])



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import security_group_rules_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSecurityGroupRulesClient(base.BaseServiceTest):

    FAKE_SECURITY_GROUP_RULE = {
        "security_group_rule": {
            "id": "2d021cf1-ce4b-4292-994f-7a785d62a144",
            "ip_range": {
                "cidr": "0.0.0.0/0"
            },
            "parent_group_id": "48700ff3-30b8-4e63-845f-a79c9633e9fb",
            "to_port": 443,
            "ip_protocol": "tcp",
            "group": {},
            "from_port": 443
        }
    }

    def setUp(self):
        super(TestSecurityGroupRulesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = security_group_rules_client.SecurityGroupRulesClient(
            fake_auth, 'compute', 'regionOne')

    def _test_create_security_group_rule(self, bytes_body=False):
        req_body = {
            "from_port": "443",
            "ip_protocol": "tcp",
            "to_port": "443",
            "cidr": "0.0.0.0/0",
            "parent_group_id": "48700ff3-30b8-4e63-845f-a79c9633e9fb"
        }
        self.check_service_client_function(
            self.client.create_security_group_rule,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SECURITY_GROUP_RULE,
            to_utf=bytes_body, **req_body)

    def test_create_security_group_rule_with_str_body(self):
        self._test_create_security_group_rule()

    def test_create_security_group_rule_with_bytes_body(self):
        self._test_create_security_group_rule(bytes_body=True)

    def test_delete_security_group_rule(self):
        self.check_service_client_function(
            self.client.delete_security_group_rule,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=202, group_rule_id='group-id')



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


list_versions = {
    'status_code': [300],
    'response_body': {
        'type': 'object',
        'properties': {
            'versions': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'status': {'type': 'string'},
                        'updated': {'type': 'string'},
                        'id': {'type': 'string'},
                        'links': {
                            'type': 'array',
                            'items': {
                                'type': 'object',
                                'properties': {
                                    'href': {'type': 'string',
                                             'format': 'uri'},
                                    'rel': {'type': 'string'},
                                    'type': {'type': 'string'},
                                },
                                'required': ['href', 'rel']
                            }
                        },
                        'min_version': {'type': 'string'},
                        'version': {'type': 'string'},
                        'media-types': {
                            'type': 'array',
                            'properties': {
                                'base': {'type': 'string'},
                                'type': {'type': 'string'}
                            },
                            'required': ['base', 'type']
                        }
                    },
                    'required': ['status', 'updated', 'id', 'links',
                                 'min_version', 'version', 'media-types']
                }
            }
        },
        'required': ['versions'],
    }
}

volume_api_version_details = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'versions': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'status': {'type': 'string'},
                        'updated': {'type': 'string'},
                        'id': {'type': 'string'},
                        'links': {
                            'type': 'array',
                            'items': {
                                'type': 'object',
                                'properties': {
                                    'href': {'type': 'string',
                                             'format': 'uri'},
                                    'rel': {'type': 'string'},
                                    'type': {'type': 'string'},
                                },
                                'required': ['href', 'rel']
                            }
                        },
                        'min_version': {'type': 'string'},
                        'version': {'type': 'string'},
                        'media-types': {
                            'type': 'array',
                            'properties': {
                                'base': {'type': 'string'},
                                'type': {'type': 'string'}
                            },
                            'required': ['base', 'type']
                        }
                    },
                    'required': ['status', 'updated', 'id', 'links',
                                 'min_version', 'version', 'media-types']
                }
            }
        },
        'required': ['versions'],
    }
}



# Copyright 2018 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from tempest.api.compute import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


class FlavorsV255TestJSON(base.BaseV2ComputeAdminTest):
    """Test flavors API with compute microversion greater than 2.54"""

    min_microversion = '2.55'
    max_microversion = 'latest'

    # NOTE(gmann): This class tests the flavors APIs
    # response schema for the 2.55 microversion.

    @decorators.idempotent_id('61976b25-488d-41dc-9dcb-cb9693a7b075')
    def test_crud_flavor(self):
        """Test create/show/update/list flavor

        Check the response schema of flavors API with microversion greater
        than 2.54.
        """
        flavor_id = data_utils.rand_int_id(start=1000)
        # Checking create API response schema
        new_flavor_id = self.create_flavor(ram=512,
                                           vcpus=1,
                                           disk=10,
                                           id=flavor_id)['id']
        # Checking show API response schema
        self.flavors_client.show_flavor(new_flavor_id)
        # Checking update API response schema
        self.admin_flavors_client.update_flavor(new_flavor_id,
                                                description='new')
        # Checking list details API response schema
        self.flavors_client.list_flavors(detail=True)
        # Checking list API response schema
        self.flavors_client.list_flavors()


class FlavorsV261TestJSON(FlavorsV255TestJSON):
    """Test flavors API with compute microversion greater than 2.60"""
    min_microversion = '2.61'
    max_microversion = 'latest'

    # NOTE(gmann): This class tests the flavors APIs
    # response schema for the 2.61 microversion.



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ServersTestJSON(base.BaseV2ComputeTest):
    """Test servers API"""
    create_default_network = True

    credentials = ['primary', 'project_reader']

    @classmethod
    def setup_clients(cls):
        super(ServersTestJSON, cls).setup_clients()
        cls.client = cls.servers_client
        if CONF.enforce_scope.nova:
            cls.reader_client = cls.os_project_reader.servers_client
        else:
            cls.reader_client = cls.client

    @decorators.idempotent_id('b92d5ec7-b1dd-44a2-87e4-45e888c46ef0')
    @testtools.skipUnless(CONF.compute_feature_enabled.
                          enable_instance_password,
                          'Instance password not available.')
    def test_create_server_with_admin_password(self):
        """Test creating server with admin password

        If an admin password is provided on server creation, the server's
        root password should be set to that password.
        """
        server = self.create_test_server(adminPass='testpassword')
        self.addCleanup(self.delete_server, server['id'])

        # Verify the password is set correctly in the response
        self.assertEqual('testpassword', server['adminPass'])

    @decorators.idempotent_id('8fea6be7-065e-47cf-89b8-496e6f96c699')
    def test_create_with_existing_server_name(self):
        """Test creating a server with already existing name is allowed"""
        # TODO(sdague): clear out try, we do cleanup one layer up
        server_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-server')
        server = self.create_test_server(name=server_name,
                                         wait_until='ACTIVE')
        id1 = server['id']
        self.addCleanup(self.delete_server, id1)
        server = self.create_test_server(name=server_name,
                                         wait_until='ACTIVE')
        id2 = server['id']
        self.addCleanup(self.delete_server, id2)
        self.assertNotEqual(id1, id2, "Did not create a new server")
        server = self.reader_client.show_server(id1)['server']
        name1 = server['name']
        server = self.reader_client.show_server(id2)['server']
        name2 = server['name']
        self.assertEqual(name1, name2)

    @decorators.idempotent_id('f9e15296-d7f9-4e62-b53f-a04e89160833')
    def test_create_specify_keypair(self):
        """Test creating server with keypair"""
        key_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='key')
        self.keypairs_client.create_keypair(name=key_name)
        self.addCleanup(self.keypairs_client.delete_keypair, key_name)
        self.keypairs_client.list_keypairs()
        server = self.create_test_server(key_name=key_name,
                                         wait_until='ACTIVE')
        self.addCleanup(self.delete_server, server['id'])
        server = self.reader_client.show_server(server['id'])['server']
        self.assertEqual(key_name, server['key_name'])

    def _update_server_name(self, server_id, status, prefix_name='server'):
        # The server name should be changed to the provided value
        new_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name=prefix_name)

        # Update the server with a new name
        self.client.update_server(server_id,
                                  name=new_name)
        waiters.wait_for_server_status(self.client, server_id, status)

        # Verify the name of the server has changed
        server = self.reader_client.show_server(server_id)['server']
        self.assertEqual(new_name, server['name'])
        return server

    @decorators.idempotent_id('5e6ccff8-349d-4852-a8b3-055df7988dd2')
    def test_update_server_name(self):
        """Test updating server name to the provided value"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.delete_server, server['id'])
        # Update instance name with non-ASCII characters
        prefix_name = '\u00CD\u00F1st\u00E1\u00F1c\u00E9'
        self._update_server_name(server['id'], 'ACTIVE', prefix_name)

        # stop server and check server name update again
        self.client.stop_server(server['id'])
        waiters.wait_for_server_status(self.client, server['id'], 'SHUTOFF')
        # Update instance name with non-ASCII characters
        updated_server = self._update_server_name(server['id'],
                                                  'SHUTOFF',
                                                  prefix_name)
        self.assertNotIn('progress', updated_server)

    @decorators.idempotent_id('89b90870-bc13-4b73-96af-f9d4f2b70077')
    def test_update_access_server_address(self):
        """Test updating server's access addresses to the provided value"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.delete_server, server['id'])

        # Update the IPv4 and IPv6 access addresses
        self.client.update_server(server['id'],
                                  accessIPv4='1.1.1.1',
                                  accessIPv6='::babe:202:202')
        waiters.wait_for_server_status(self.client, server['id'], 'ACTIVE')

        # Verify the access addresses have been updated
        server = self.reader_client.show_server(server['id'])['server']
        self.assertEqual('1.1.1.1', server['accessIPv4'])
        self.assertEqual('::babe:202:202', server['accessIPv6'])

    @decorators.idempotent_id('38fb1d02-c3c5-41de-91d3-9bc2025a75eb')
    def test_create_server_with_ipv6_addr_only(self):
        """Test creating server with ipv6 address only(no ipv4 address)"""
        server = self.create_test_server(accessIPv6='2001:2001::3',
                                         wait_until='ACTIVE')
        self.addCleanup(self.delete_server, server['id'])
        server = self.reader_client.show_server(server['id'])['server']
        self.assertEqual('2001:2001::3', server['accessIPv6'])

    @decorators.related_bug('1730756')
    @decorators.idempotent_id('defbaca5-d611-49f5-ae21-56ee25d2db49')
    def test_create_server_specify_multibyte_character_name(self):
        """Test creating server with multi character name

        prefix character is:
        http://unicode.org/cldr/utility/character.jsp?a=20A1

        We use a string with 3 byte utf-8 character due to nova
        will return 400(Bad Request) if we attempt to send a name which has
        4 byte utf-8 character.
        """
        utf8_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=b'\xe2\x82\xa1'.decode('utf-8'))
        self.create_test_server(name=utf8_name, wait_until='ACTIVE')


class ServerShowV247Test(base.BaseV2ComputeTest):
    """Test servers API with compute microversion greater than 2.46"""

    min_microversion = '2.47'
    max_microversion = 'latest'

    # NOTE(gmann): This test tests the server APIs response schema
    # Along with 2.47 microversion schema this test class tests the
    # other microversions 2.9, 2.19 and 2.26 server APIs response schema
    # also. 2.47 APIs schema are on top of 2.9->2.19->2.26 schema so
    # below tests cover all of the schema.

    credentials = ['primary', 'project_reader']

    @classmethod
    def setup_clients(cls):
        super(ServerShowV247Test, cls).setup_clients()
        if CONF.enforce_scope.nova:
            cls.reader_client = cls.os_project_reader.servers_client
        else:
            cls.reader_client = cls.servers_client

    @decorators.idempotent_id('88b0bdb2-494c-11e7-a919-92ebcb67fe33')
    def test_show_server(self):
        """Test getting server detail"""
        server = self.create_test_server()
        # All fields will be checked by API schema
        self.reader_client.show_server(server['id'])

    @decorators.idempotent_id('8de397c2-57d0-4b90-aa30-e5d668f21a8b')
    def test_update_rebuild_list_server(self):
        """Test update/rebuild/list server"""
        server = self.create_test_server()
        # Checking update API response schema
        self.servers_client.update_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')
        # Checking rebuild API response schema
        self.servers_client.rebuild_server(server['id'], self.image_ref_alt)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')
        # Checking list details API response schema
        self.servers_client.list_servers(detail=True)


class ServerShowV263Test(base.BaseV2ComputeTest):
    """Test servers API with compute microversion greater than 2.62"""

    min_microversion = '2.63'
    max_microversion = 'latest'

    credentials = ['primary', 'project_reader']

    @classmethod
    def setup_clients(cls):
        super(ServerShowV263Test, cls).setup_clients()
        if CONF.enforce_scope.nova:
            cls.reader_client = cls.os_project_reader.servers_client
        else:
            cls.reader_client = cls.servers_client

    @testtools.skipUnless(CONF.compute.certified_image_ref,
                          '``[compute]/certified_image_ref`` required to test '
                          'image certificate validation.')
    @testtools.skipUnless(CONF.compute.certified_image_trusted_certs,
                          '``[compute]/certified_image_trusted_certs`` '
                          'required to test image certificate validation.')
    @decorators.idempotent_id('71b8e3d5-11d2-494f-b917-b094a4afed3c')
    def test_show_update_rebuild_list_server(self):
        """Test show/update/rebuild/list server"""
        trusted_certs = CONF.compute.certified_image_trusted_certs
        server = self.create_test_server(
            image_id=CONF.compute.certified_image_ref,
            trusted_image_certificates=trusted_certs,
            wait_until='ACTIVE')

        # Check show API response schema
        self.reader_client.show_server(server['id'])['server']

        # Check update API response schema
        self.servers_client.update_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')

        # Check rebuild API response schema
        self.servers_client.rebuild_server(
            server['id'], CONF.compute.certified_image_ref)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')

        # Check list details API response schema
        params = {'trusted_image_certificates': trusted_certs}
        servers = self.servers_client.list_servers(
            detail=True, **params)['servers']
        self.assertNotEmpty(servers)


class ServersListShow296Test(base.BaseV2ComputeTest):
    """Test compute server with microversion >= than 2.96

    This test tests the Server APIs response schema for 2.96 microversion.
    No specific assert or behaviour verification is needed.
    """

    min_microversion = '2.96'
    max_microversion = 'latest'

    @decorators.idempotent_id('4eee1ffe-9e00-4c99-a431-0d3e0f323a8f')
    def test_list_show_update_rebuild_server_296(self):
        server = self.create_test_server(wait_until='ACTIVE')
        # Checking list API response schema.
        self.servers_client.list_servers(detail=True)
        # Checking show API response schema
        self.servers_client.show_server(server['id'])
        # Checking update API response schema
        self.servers_client.update_server(server['id'])
        # Check rebuild API response schema
        self.servers_client.rebuild_server(server['id'], self.image_ref_alt)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')


class ServersListShow298Test(base.BaseV2ComputeTest):
    """Test compute server with microversion >= 2.98"""

    min_microversion = '2.98'
    max_microversion = 'latest'

    @decorators.idempotent_id('3981e496-3bf7-4015-b807-63ffee7c520c')
    def test_list_show_update_rebuild_server_298(self):
        server = self.create_test_server(wait_until='ACTIVE')
        # Check list details API response schema
        self.servers_client.list_servers(detail=True)
        # Check show API response schema
        self.servers_client.show_server(server['id'])
        # Checking update API response schema
        self.servers_client.update_server(server['id'])
        # Check rebuild API response schema
        self.servers_client.rebuild_server(server['id'], self.image_ref_alt)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')


class ServersListShow2100Test(base.BaseV2ComputeTest):
    """Test compute server with microversion >= than 2.100

    This test tests the Server APIs response schema for 2.100 microversion.
    No specific assert or behaviour verification is needed.
    """

    min_microversion = '2.100'
    max_microversion = 'latest'

    @decorators.idempotent_id('2c3a8270-e6f7-4400-af0f-db003c117e48')
    def test_list_show_rebuild_update_server_2100(self):
        server = self.create_test_server(wait_until='ACTIVE')
        # Checking list API response schema.
        self.servers_client.list_servers(detail=True)
        # Checking show API response schema
        self.servers_client.show_server(server['id'])
        # Checking update API response schema
        self.servers_client.update_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')
        # Check rebuild API response schema
        self.servers_client.rebuild_server(server['id'], self.image_ref_alt)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')



# Copyright 2013 IBM Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class FlavorsAccessNegativeTestJSON(base.BaseV2ComputeAdminTest):
    """Tests Flavor Access API extension.

    Add and remove Flavor Access require admin privileges.
    """

    credentials = ['primary', 'admin', 'alt']

    @classmethod
    def resource_setup(cls):
        super(FlavorsAccessNegativeTestJSON, cls).resource_setup()

        cls.tenant_id = cls.flavors_client.tenant_id
        cls.ram = 512
        cls.vcpus = 1
        cls.disk = 10

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0621c53e-d45d-40e7-951d-43e5e257b272')
    def test_flavor_access_list_with_public_flavor(self):
        """Test listing flavor access of a public flavor should fail"""
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk, is_public='True')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_flavors_client.list_flavor_access,
                          flavor['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('41eaaade-6d37-4f28-9c74-f21b46ca67bd')
    def test_flavor_non_admin_add(self):
        """Test adding flavor access by a non-admin user is forbidden"""
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk, is_public='False')
        self.assertRaises(lib_exc.Forbidden,
                          self.flavors_client.add_flavor_access,
                          flavor['id'],
                          self.tenant_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('073e79a6-c311-4525-82dc-6083d919cb3a')
    def test_flavor_non_admin_remove(self):
        """Test removing flavor access by a non-admin user should fail"""
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk, is_public='False')

        # Add flavor access to a tenant.
        self.admin_flavors_client.add_flavor_access(flavor['id'],
                                                    self.tenant_id)
        self.addCleanup(self.admin_flavors_client.remove_flavor_access,
                        flavor['id'], self.tenant_id)
        self.assertRaises(lib_exc.Forbidden,
                          self.flavors_client.remove_flavor_access,
                          flavor['id'],
                          self.tenant_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f3592cc0-0306-483c-b210-9a7b5346eddc')
    def test_add_flavor_access_duplicate(self):
        """Test adding duplicate flavor access to same flavor should fail"""
        # Create a new flavor.
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk, is_public='False')

        # Add flavor access to a tenant.
        self.admin_flavors_client.add_flavor_access(flavor['id'],
                                                    self.tenant_id)
        self.addCleanup(self.admin_flavors_client.remove_flavor_access,
                        flavor['id'], self.tenant_id)

        # An exception should be raised when adding flavor access to the same
        # tenant
        self.assertRaises(lib_exc.Conflict,
                          self.admin_flavors_client.add_flavor_access,
                          flavor['id'],
                          self.tenant_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1f710927-3bc7-4381-9f82-0ca6e42644b7')
    def test_remove_flavor_access_not_found(self):
        """Test removing non existent flavor access should fail"""
        # Create a new flavor.
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk, is_public='False')

        # An exception should be raised when flavor access is not found
        self.assertRaises(lib_exc.NotFound,
                          self.admin_flavors_client.remove_flavor_access,
                          flavor['id'],
                          self.os_alt.servers_client.tenant_id)



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

from oslo_serialization import jsonutils as json

from tempest.lib.services.network import base as network_base
from tempest.lib.services.network import security_group_rules_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSecurityGroupsClient(base.BaseServiceTest):

    FAKE_SEC_GROUP_RULE_ID = "3c0e45ff-adaf-4124-b083-bf390e5482ff"

    FAKE_SECURITY_GROUP_RULES = {
        "security_group_rules": [
            {
                "direction": "egress",
                "ethertype": "IPv6",
                "id": "3c0e45ff-adaf-4124-b083-bf390e5482ff",
                "port_range_max": None,
                "port_range_min": None,
                "protocol": None,
                "remote_group_id": None,
                "remote_ip_prefix": None,
                "security_group_id": "85cc3048-abc3-43cc-89b3-377341426ac5",
                "project_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                "tenant_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                "description": ""
            },
            {
                "direction": "egress",
                "ethertype": "IPv4",
                "id": "93aa42e5-80db-4581-9391-3a608bd0e448",
                "port_range_max": None,
                "port_range_min": None,
                "protocol": None,
                "remote_group_id": None,
                "remote_ip_prefix": None,
                "security_group_id": "85cc3048-abc3-43cc-89b3-377341426ac5",
                "project_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                "tenant_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                "description": ""
            }
        ]
    }

    FAKE_SECURITY_GROUP_RULE = copy.copy(
        FAKE_SECURITY_GROUP_RULES['security_group_rules'][0])

    def setUp(self):
        super(TestSecurityGroupsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = security_group_rules_client.SecurityGroupRulesClient(
            fake_auth, 'network', 'regionOne')

    def _test_list_security_group_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_security_group_rules,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SECURITY_GROUP_RULES,
            bytes_body,
            mock_args='v2.0/security-group-rules')

    def _test_create_security_group_rule(self, bytes_body=False):
        kwargs = {'direction': 'egress',
                  'security_group_id': '85cc3048-abc3-43cc-89b3-377341426ac5',
                  'remote_ip_prefix': None}
        payload = json.dumps({"security_group_rule": kwargs}, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(network_base.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.create_security_group_rule,
                'tempest.lib.common.rest_client.RestClient.post',
                self.FAKE_SECURITY_GROUP_RULE,
                bytes_body,
                status=201,
                mock_args=['v2.0/security-group-rules', payload],
                **kwargs)

    def _test_show_security_group_rule(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_security_group_rule,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SECURITY_GROUP_RULE,
            bytes_body,
            security_group_rule_id=self.FAKE_SEC_GROUP_RULE_ID,
            mock_args='v2.0/security-group-rules/%s'
                      % self.FAKE_SEC_GROUP_RULE_ID)

    def test_list_security_group_rules_with_str_body(self):
        self._test_list_security_group_rules()

    def test_list_security_group_rules_with_bytes_body(self):
        self._test_list_security_group_rules(bytes_body=True)

    def test_create_security_group_rule_with_str_body(self):
        self._test_create_security_group_rule()

    def test_create_security_group_rule_with_bytes_body(self):
        self._test_create_security_group_rule(bytes_body=True)

    def test_show_security_group_rule_with_str_body(self):
        self._test_show_security_group_rule()

    def test_show_security_group_rule_with_bytes_body(self):
        self._test_show_security_group_rule(bytes_body=True)

    def test_delete_security_group_rule(self):
        self.check_service_client_function(
            self.client.delete_security_group_rule,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            security_group_rule_id=self.FAKE_SEC_GROUP_RULE_ID,
            mock_args='v2.0/security-group-rules/%s'
                      % self.FAKE_SEC_GROUP_RULE_ID)



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io
import time

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions
import tempest.test

CONF = config.CONF
BAD_REQUEST_RETRIES = 3


class BaseImageTest(tempest.test.BaseTestCase):
    """Base test class for Image API tests."""

    credentials = ['primary']

    @classmethod
    def skip_checks(cls):
        super(BaseImageTest, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources()
        super(BaseImageTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(BaseImageTest, cls).resource_setup()
        cls.created_images = []

    @classmethod
    def create_image(cls, data=None, **kwargs):
        """Wrapper that returns a test image."""

        if 'name' not in kwargs:
            name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=cls.__name__ + "-image")
            kwargs['name'] = name

        image = cls.client.create_image(**kwargs)
        cls.created_images.append(image['id'])
        cls.addClassResourceCleanup(cls.client.wait_for_resource_deletion,
                                    image['id'])
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.client.delete_image, image['id'])
        return image


class BaseV2ImageTest(BaseImageTest):

    @classmethod
    def skip_checks(cls):
        super(BaseV2ImageTest, cls).skip_checks()
        if not CONF.image_feature_enabled.api_v2:
            msg = "Glance API v2 not supported"
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(BaseV2ImageTest, cls).setup_clients()
        cls.client = cls.os_primary.image_client_v2
        cls.schemas_client = cls.os_primary.schemas_client
        cls.versions_client = cls.os_primary.image_versions_client

    def create_namespace(self, namespace_name=None, visibility='public',
                         description='Tempest', protected=False,
                         **kwargs):
        if not namespace_name:
            namespace_name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix, name='test-ns')
        kwargs.setdefault('display_name', namespace_name)
        namespace = self.namespaces_client.create_namespace(
            namespace=namespace_name, visibility=visibility,
            description=description, protected=protected, **kwargs)
        self.addCleanup(self.namespaces_client.delete_namespace,
                        namespace_name)
        return namespace

    def create_and_stage_image(self, all_stores=False):
        """Create Image & stage image file for glance-direct import method."""
        image_name = data_utils.rand_name('test-image')
        container_format = CONF.image.container_formats[0]
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format='raw',
                                  visibility='private')
        self.assertEqual('queued', image['status'])

        self.client.stage_image_file(
            image['id'],
            io.BytesIO(data_utils.random_bytes()))
        # Check image status is 'uploading'
        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual('uploading', body['status'])

        if all_stores:
            stores_list = ','.join([store['id']
                                    for store in self.available_stores
                                    if store.get('read-only') != 'true'])
        else:
            stores = [store['id'] for store in self.available_stores
                      if store.get('read-only') != 'true']
            stores_list = stores[::max(1, len(stores) - 1)]

        return body, stores_list

    @classmethod
    def get_available_stores(cls):
        stores = []
        try:
            stores = cls.client.info_stores()['stores']
        except exceptions.NotFound:
            pass
        return stores

    def _update_image_with_retries(self, image, patch):
        # NOTE(danms): If glance was unable to fetch the remote image via
        # HTTP, it will return BadRequest. Because this can be transient in
        # CI, we try this a few times before we agree that it has failed
        # for a reason worthy of failing the test.
        for i in range(BAD_REQUEST_RETRIES):
            try:
                self.client.update_image(image, patch)
                break
            except exceptions.BadRequest:
                if i + 1 == BAD_REQUEST_RETRIES:
                    raise
                else:
                    time.sleep(1)

    def check_set_location(self):
        image = self.client.create_image(container_format='bare',
                                         disk_format='raw')

        # Locations should be empty when there is no data
        self.assertEqual('queued', image['status'])
        self.assertEqual([], image['locations'])

        # Add a new location
        new_loc = {'metadata': {'foo': 'bar'},
                   'url': CONF.image.http_image}
        self._update_image_with_retries(image['id'], [
            dict(add='/locations/-', value=new_loc)])

        # The image should now be active, with one location that looks
        # like we expect
        image = self.client.show_image(image['id'])
        self.assertEqual(1, len(image['locations']),
                         'Image should have one location but has %i' % (
                         len(image['locations'])))
        self.assertEqual(new_loc['url'], image['locations'][0]['url'])
        self.assertEqual('bar', image['locations'][0]['metadata'].get('foo'))
        if 'direct_url' in image:
            self.assertEqual(image['direct_url'], image['locations'][0]['url'])

        # If we added the location directly, the image goes straight
        # to active and no hashing is done
        self.assertEqual('active', image['status'])
        self.assertIsNone(image['os_hash_algo'])
        self.assertIsNone(image['os_hash_value'])

        return image

    def check_set_multiple_locations(self):
        image = self.check_set_location()

        new_loc = {'metadata': {'speed': '88mph'},
                   'url': '%s#new' % CONF.image.http_image}
        self._update_image_with_retries(image['id'],
                                        [dict(add='/locations/-',
                                              value=new_loc)])

        # The image should now have two locations and the last one
        # (locations are ordered) should have the new URL.
        image = self.client.show_image(image['id'])
        self.assertEqual(2, len(image['locations']),
                         'Image should have two locations but has %i' % (
                         len(image['locations'])))
        self.assertEqual(new_loc['url'], image['locations'][1]['url'])

        # The image should still be active and still have no hashes
        self.assertEqual('active', image['status'])
        self.assertIsNone(image['os_hash_algo'])
        self.assertIsNone(image['os_hash_value'])

        # The direct_url should still match the first location
        if 'direct_url' in image:
            self.assertEqual(image['direct_url'], image['locations'][0]['url'])

        return image


class BaseV2MemberImageTest(BaseV2ImageTest):

    credentials = ['primary', 'alt']

    @classmethod
    def setup_clients(cls):
        super(BaseV2MemberImageTest, cls).setup_clients()
        cls.image_member_client = cls.os_primary.image_member_client_v2
        cls.alt_image_member_client = cls.os_alt.image_member_client_v2
        cls.alt_img_client = cls.os_alt.image_client_v2

    @classmethod
    def resource_setup(cls):
        super(BaseV2MemberImageTest, cls).resource_setup()
        cls.alt_tenant_id = cls.alt_image_member_client.tenant_id

    def _list_image_ids_as_alt(self):
        image_list = self.alt_img_client.list_images()['images']
        image_ids = map(lambda x: x['id'], image_list)
        return image_ids

    def _create_image(self):
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-image')
        image = self.client.create_image(name=name,
                                         container_format='bare',
                                         disk_format='raw')
        self.addCleanup(self.client.delete_image, image['id'])
        return image['id']


class BaseV2ImageAdminTest(BaseV2ImageTest):

    credentials = ['admin', 'primary']

    @classmethod
    def setup_clients(cls):
        super(BaseV2ImageAdminTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.image_client_v2
        cls.namespaces_client = cls.os_admin.namespaces_client
        cls.resource_types_client = cls.os_admin.resource_types_client
        cls.namespace_properties_client =\
            cls.os_admin.namespace_properties_client
        cls.namespace_objects_client = cls.os_admin.namespace_objects_client
        cls.namespace_tags_client = cls.os_admin.namespace_tags_client



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Manages Tempest workspaces

This command is used for managing tempest workspaces

Commands
========

list
----
Outputs the name and path of all known tempest workspaces

register
--------
Registers a new tempest workspace via a given ``--name`` and ``--path``

rename
------
Renames a tempest workspace from ``--old-name`` to ``--new-name``

move
----
Changes the path of a given tempest workspace ``--name`` to ``--path``

remove
------
Deletes the entry for a given tempest workspace ``--name``

``--rmdir`` Deletes the given tempest workspace directory

General Options
===============

* ``--workspace_path``: Allows the user to specify a different location for the
  workspace.yaml file containing the workspace definitions instead of
  ``~/.tempest/workspace.yaml``
"""

import os
import shutil
import sys

from cliff import command
from cliff import lister
from oslo_concurrency import lockutils
import yaml

from tempest import config

CONF = config.CONF


class WorkspaceManager(object):
    def __init__(self, path=None):
        lockutils.get_lock_path(CONF)
        self.path = path or os.path.join(
            os.path.expanduser("~"), ".tempest", "workspace.yaml")
        if not os.path.isdir(os.path.dirname(self.path)):
            os.makedirs(self.path.rsplit(os.path.sep, 1)[0])
        self.workspaces = {}

    @lockutils.synchronized('workspaces', external=True)
    def get_workspace(self, name):
        """Returns the workspace that has the given name

        If the workspace isn't registered then `None` is returned.
        """
        self._populate()
        return self.workspaces.get(name)

    @lockutils.synchronized('workspaces', external=True)
    def rename_workspace(self, old_name, new_name):
        self._populate()
        self._name_exists(old_name)
        self._invalid_name_check(new_name)
        self._workspace_name_exists(new_name)
        self.workspaces[new_name] = self.workspaces.pop(old_name)
        self._write_file()

    @lockutils.synchronized('workspaces', external=True)
    def move_workspace(self, name, path):
        self._populate()
        path = os.path.abspath(os.path.expanduser(path)) if path else path
        self._name_exists(name)
        self._validate_path(path)
        self.workspaces[name] = path
        self._write_file()

    def _name_exists(self, name):
        if name not in self.workspaces:
            print("A workspace was not found with name: {0}".format(name))
            sys.exit(1)

    @lockutils.synchronized('workspaces', external=True)
    def remove_workspace_entry(self, name):
        self._populate()
        self._name_exists(name)
        workspace_path = self.workspaces.pop(name)
        self._write_file()
        return workspace_path

    @lockutils.synchronized('workspaces', external=True)
    def remove_workspace_directory(self, workspace_path):
        self._validate_path(workspace_path)
        shutil.rmtree(workspace_path)

    @lockutils.synchronized('workspaces', external=True)
    def list_workspaces(self):
        self._populate()
        self._validate_workspaces()
        return self.workspaces

    def _workspace_name_exists(self, name):
        if name in self.workspaces:
            print("A workspace already exists with name: {0}.".format(
                name))
            sys.exit(1)

    def _invalid_name_check(self, name):
        if not name:
            print("None or empty name is specified."
                  " Please specify correct name for workspace.")
            sys.exit(1)

    def _validate_path(self, path):
        if not path:
            print("None or empty path is specified for workspace."
                  " Please specify correct workspace path.")
            sys.exit(1)
        if not os.path.exists(path):
            print("Path does not exist.")
            sys.exit(1)

    @lockutils.synchronized('workspaces', external=True)
    def register_new_workspace(self, name, path, init=False):
        """Adds the new workspace and writes out the new workspace config"""
        self._populate()
        path = os.path.abspath(os.path.expanduser(path)) if path else path
        # This only happens when register is called from outside of init
        if not init:
            self._validate_path(path)
        self._invalid_name_check(name)
        self._workspace_name_exists(name)
        self.workspaces[name] = path
        self._write_file()

    def _validate_workspaces(self):
        if self.workspaces is not None:
            self.workspaces = {n: p for n, p in self.workspaces.items()
                               if os.path.exists(p)}
            self._write_file()

    def _write_file(self):
        with open(self.path, 'w') as f:
            f.write(yaml.dump(self.workspaces))

    def _populate(self):
        if not os.path.isfile(self.path):
            return
        with open(self.path, 'r') as f:
            self.workspaces = yaml.safe_load(f) or {}


def add_global_arguments(parser):
    parser.add_argument(
        '--workspace-path', required=False, default=None,
        help="The path to the workspace file, the default is "
             "~/.tempest/workspace.yaml")
    return parser


class TempestWorkspaceRegister(command.Command):
    def get_description(self):
        return ('Registers a new tempest workspace via a given '
                '--name and --path')

    def get_parser(self, prog_name):
        parser = super(TempestWorkspaceRegister, self).get_parser(prog_name)
        add_global_arguments(parser)
        parser.add_argument('--name', required=True)
        parser.add_argument('--path', required=True)

        return parser

    def take_action(self, parsed_args):
        self.manager = WorkspaceManager(parsed_args.workspace_path)
        self.manager.register_new_workspace(parsed_args.name, parsed_args.path)
        sys.exit(0)


class TempestWorkspaceRename(command.Command):
    def get_description(self):
        return 'Renames a tempest workspace from --old-name to --new-name'

    def get_parser(self, prog_name):
        parser = super(TempestWorkspaceRename, self).get_parser(prog_name)
        add_global_arguments(parser)
        parser.add_argument('--old-name', required=True)
        parser.add_argument('--new-name', required=True)

        return parser

    def take_action(self, parsed_args):
        self.manager = WorkspaceManager(parsed_args.workspace_path)
        self.manager.rename_workspace(
            parsed_args.old_name, parsed_args.new_name)
        sys.exit(0)


class TempestWorkspaceMove(command.Command):
    def get_description(self):
        return 'Changes the path of a given tempest workspace --name to --path'

    def get_parser(self, prog_name):
        parser = super(TempestWorkspaceMove, self).get_parser(prog_name)
        add_global_arguments(parser)
        parser.add_argument('--name', required=True)
        parser.add_argument('--path', required=True)

        return parser

    def take_action(self, parsed_args):
        self.manager = WorkspaceManager(parsed_args.workspace_path)
        self.manager.move_workspace(parsed_args.name, parsed_args.path)
        sys.exit(0)


class TempestWorkspaceRemove(command.Command):
    def get_description(self):
        return 'Deletes the entry for a given tempest workspace --name'

    def get_parser(self, prog_name):
        parser = super(TempestWorkspaceRemove, self).get_parser(prog_name)
        add_global_arguments(parser)
        parser.add_argument('--name', required=True)
        parser.add_argument('--rmdir', action='store_true',
                            help='Deletes the given workspace directory')

        return parser

    def take_action(self, parsed_args):
        self.manager = WorkspaceManager(parsed_args.workspace_path)
        workspace_path = self.manager.remove_workspace_entry(parsed_args.name)
        if parsed_args.rmdir:
            self.manager.remove_workspace_directory(workspace_path)
        sys.exit(0)


class TempestWorkspaceList(lister.Lister):
    def get_description(self):
        return 'Outputs the name and path of all known tempest workspaces'

    def get_parser(self, prog_name):
        parser = super(TempestWorkspaceList, self).get_parser(prog_name)
        add_global_arguments(parser)
        return parser

    def take_action(self, parsed_args):
        self.manager = WorkspaceManager(parsed_args.workspace_path)
        return (("Name", "Path"),
                ((n, p) for n, p in self.manager.list_workspaces().items()))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import \
    security_groups as schema
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class SecurityGroupRulesClient(base_compute_client.BaseComputeClient):

    def create_security_group_rule(self, **kwargs):
        """Create a new security group rule.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-security-group-rule
        """
        post_body = json.dumps({'security_group_rule': kwargs})
        url = 'os-security-group-rules'
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        self.validate_response(schema.create_security_group_rule, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_security_group_rule(self, group_rule_id):
        """Deletes the provided Security Group rule."""
        resp, body = self.delete('os-security-group-rules/%s' %
                                 group_rule_id)
        self.validate_response(schema.delete_security_group_rule, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 IBM Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import \
    instance_usage_audit_logs as schema
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class InstanceUsagesAuditLogClient(base_compute_client.BaseComputeClient):

    def list_instance_usage_audit_logs(self):
        url = 'os-instance_usage_audit_log'
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_instance_usage_audit_log,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_instance_usage_audit_log(self, time_before):
        url = 'os-instance_usage_audit_log/%s' % time_before
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.get_instance_usage_audit_log, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2021 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import floating_ips_port_forwarding_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestFloatingIpsPortForwardingClient(base.BaseServiceTest):

    FAKE_PORT_FORWARDING_REQUEST = {

        "port_forwarding": {
            "protocol": "tcp",
            "internal_ip_address": "10.0.0.11",
            "internal_port": 25,
            "internal_port_id": "1238be08-a2a8-4b8d-addf-fb5e2250e480",
            "external_port": 2230,
            "description": "Some description",
            }

        }

    FAKE_PORT_FORWARDING_RESPONSE = {

        "port_forwarding": {
            "protocol": "tcp",
            "internal_ip_address": "10.0.0.12",
            "internal_port": 26,
            "internal_port_id": "1238be08-a2a8-4b8d-addf-fb5e2250e480",
            "external_port": 2130,
            "description": "Some description",
            "id": "825ade3c-9760-4880-8080-8fc2dbab9acc"
        }
    }

    FAKE_PORT_FORWARDINGS = {
        "port_forwardings": [
            FAKE_PORT_FORWARDING_RESPONSE['port_forwarding']
        ]
    }

    FAKE_FLOATINGIP_ID = "a6800594-5b7a-4105-8bfe-723b346ce866"

    FAKE_PORT_FORWARDING_ID = "a7800594-5b7a-4105-8bfe-723b346ce866"

    def setUp(self):
        super(TestFloatingIpsPortForwardingClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.floating_ips_port_forwarding_client = \
            floating_ips_port_forwarding_client.\
            FloatingIpsPortForwardingClient(fake_auth,
                                            "network",
                                            "regionOne")

    def _test_create_port_forwarding(self, bytes_body=False):
        self.check_service_client_function(
            self.floating_ips_port_forwarding_client.
            create_port_forwarding,
            "tempest.lib.common.rest_client.RestClient.post",
            self.FAKE_PORT_FORWARDING_RESPONSE,
            bytes_body,
            201,
            floatingip_id=self.FAKE_FLOATINGIP_ID,
            **self.FAKE_PORT_FORWARDING_REQUEST)

    def _test_list_port_forwardings(self, bytes_body=False):
        self.check_service_client_function(
            self.floating_ips_port_forwarding_client.
            list_port_forwardings,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_PORT_FORWARDINGS,
            bytes_body,
            200,
            floatingip_id=self.FAKE_FLOATINGIP_ID)

    def _test_show_port_forwardings(self, bytes_body=False):
        self.check_service_client_function(
            self.floating_ips_port_forwarding_client.
            show_port_forwarding,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_PORT_FORWARDING_RESPONSE,
            bytes_body,
            200,
            floatingip_id=self.FAKE_FLOATINGIP_ID,
            port_forwarding_id=self.FAKE_PORT_FORWARDING_ID)

    def _test_delete_port_forwarding(self):
        self.check_service_client_function(
            self.floating_ips_port_forwarding_client.
            delete_port_forwarding,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            floatingip_id=self.FAKE_FLOATINGIP_ID,
            port_forwarding_id=self.FAKE_PORT_FORWARDING_ID)

    def _test_update_port_forwarding(self, bytes_body=False):
        update_kwargs = {
            "internal_port": "27"
        }

        resp_body = {
            "port_forwarding": copy.deepcopy(
                self.FAKE_PORT_FORWARDING_RESPONSE['port_forwarding']
            )
        }
        resp_body["port_forwarding"].update(update_kwargs)

        self.check_service_client_function(
            self.floating_ips_port_forwarding_client.update_port_forwarding,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            floatingip_id=self.FAKE_FLOATINGIP_ID,
            port_forwarding_id=self.FAKE_PORT_FORWARDING_ID,
            **update_kwargs)

    def test_list_port_forwardings_with_str_body(self):
        self._test_list_port_forwardings()

    def test_list_port_forwardings_with_bytes_body(self):
        self._test_list_port_forwardings(bytes_body=True)

    def test_show_port_forwardings_with_str_body(self):
        self._test_show_port_forwardings()

    def test_show_port_forwardings_with_bytes_body(self):
        self._test_show_port_forwardings(bytes_body=True)

    def test_create_port_forwarding_with_str_body(self):
        self._test_create_port_forwarding()

    def test_create_port_forwarding_with_bytes_body(self):
        self._test_create_port_forwarding(bytes_body=True)

    def test_update_port_forwarding_with_str_body(self):
        self._test_update_port_forwarding()

    def test_update_port_forwarding_with_bytes_body(self):
        self._test_update_port_forwarding(bytes_body=True)



# Copyright 2024 Openstack Foundation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import time

from oslo_log import log as logging

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
from tempest.scenario import manager


CONF = config.CONF
LOG = logging.getLogger(__name__)


class TestInstancesWithCinderVolumes(manager.ScenarioTest):
    """This is cinder volumes test.

    Tests are below:
    * test_instances_with_cinder_volumes_on_all_compute_nodes
    """

    compute_min_microversion = '2.60'

    @decorators.idempotent_id('d0e3c1a3-4b0a-4b0e-8b0a-4b0e8b0a4b0e')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'volume', 'image', 'network')
    def test_instances_with_cinder_volumes_on_all_compute_nodes(self):
        """Test instances with cinder volumes launches on all compute nodes

        Steps:
            1. Create an image
            2. Create a keypair
            3. Create a bootable volume from the image and of the given volume
               type
            4. Boot an instance from the bootable volume on each available
               compute node, up to CONF.compute.min_compute_nodes
            5. Create a volume using each volume_types_for_data_volume on all
               available compute nodes, up to CONF.compute.min_compute_nodes.
               Total number of volumes is equal to
               compute nodes * len(volume_types_for_data_volume)
            6. Assign floating IP to all instances
            7. Configure security group for ssh access to all instances
            8. Confirm ssh access to all instances
            9. Attach volumes to the instances; fixup device mapping if
               required
            10. Run write test to all volumes through ssh connection per
                instance
            11. Clean up the sources, an instance, volumes, keypair and image
        """
        boot_volume_type = (CONF.volume.volume_type or
                            self.create_volume_type()['name'])

        # create an image
        image = self.image_create()

        # create keypair
        keypair = self.create_keypair()

        # check all available zones for booting instances
        available_zone = \
            self.os_admin.availability_zone_client.list_availability_zones(
                detail=True)['availabilityZoneInfo']

        hosts = []
        for zone in available_zone:
            if zone['zoneState']['available']:
                for host in zone['hosts']:
                    if 'nova-compute' in zone['hosts'][host] and \
                        zone['hosts'][host]['nova-compute']['available'] and \
                        CONF.compute.target_hosts_to_avoid not in host:
                        hosts.append({'zone': zone['zoneName'],
                                      'host_name': host})

        # fail if there is less hosts than minimal number of instances
        if len(hosts) < CONF.compute.min_compute_nodes:
            raise exceptions.InvalidConfiguration(
                "Host list %s is shorter than min_compute_nodes. " % hosts)

        # get volume types
        volume_types = []
        if CONF.volume_feature_enabled.volume_types_for_data_volume:
            types = CONF.volume_feature_enabled.volume_types_for_data_volume
            volume_types = types.split(',')
        else:
            # no user specified volume types, create 2 default ones
            volume_types.append(self.create_volume_type()['name'])
            volume_types.append(self.create_volume_type()['name'])

        hosts_to_boot_servers = hosts[:CONF.compute.min_compute_nodes]
        LOG.debug("List of hosts selected to boot servers %s: ",
                  hosts_to_boot_servers)

        # create volumes so that we dont need to wait for them to be created
        # and save them in a list
        created_volumes = []
        for host in hosts_to_boot_servers:
            for volume_type in volume_types:
                created_volumes.append(
                    self.create_volume(volume_type=volume_type,
                                       wait_until=None)
                )

        bootable_volumes = []
        for host in hosts_to_boot_servers:
            # create boot volume from image and of the given volume type
            bootable_volumes.append(
                self.create_volume(
                    imageRef=image, volume_type=boot_volume_type,
                    wait_until=None)
            )

        # boot server
        servers = []

        for bootable_volume in bootable_volumes:

            # wait for bootable volumes to become available
            waiters.wait_for_volume_resource_status(
                self.volumes_client, bootable_volume['id'], 'available')

            # create an instance from bootable volume
            server = self.boot_instance_from_resource(
                source_id=bootable_volume['id'],
                source_type='volume',
                keypair=keypair,
                wait_until=None
            )
            servers.append(server)

        start = 0
        end = len(volume_types)
        for server in servers:
            # wait for server to become active
            waiters.wait_for_server_status(self.servers_client,
                                           server['id'], 'ACTIVE')

            # assign floating ip
            floating_ip = None
            if (CONF.network_feature_enabled.floating_ips and
                CONF.network.floating_network_name):
                fip = self.create_floating_ip(server)
                floating_ip = self.associate_floating_ip(
                    fip, server)
                ssh_ip = floating_ip['floating_ip_address']
            else:
                ssh_ip = self.get_server_ip(server)

            # create security group
            self.create_and_add_security_group_to_server(server)

            # confirm ssh access
            self.linux_client = self.get_remote_client(
                ssh_ip, private_key=keypair['private_key'],
                server=server
            )

            # attach volumes to the instances
            attached_volumes = []
            for volume in created_volumes[start:end]:
                attached_volume, actual_dev = self._attach_fixup(
                    server, volume)
                attached_volumes.append((attached_volume, actual_dev))
                LOG.debug("Attached volume %s to server %s",
                          attached_volume['id'], server['id'])

            server_name = server['name'].split('-')[-1]

            # run write test on all volumes
            for volume, dev_name in attached_volumes:
                mount_path = f"/mnt/{server_name}"

                timestamp_before = self.create_timestamp(
                    ssh_ip, private_key=keypair['private_key'], server=server,
                    dev_name=dev_name, mount_path=mount_path,
                )
                timestamp_after = self.get_timestamp(
                    ssh_ip, private_key=keypair['private_key'], server=server,
                    dev_name=dev_name, mount_path=mount_path,
                )
                self.assertEqual(timestamp_before, timestamp_after)

                # delete volume
                self.nova_volume_detach(server, volume)
                self.volumes_client.delete_volume(volume['id'])

            if floating_ip:
                # delete the floating IP, this should refresh the server
                # addresses
                self.disassociate_floating_ip(floating_ip)
                waiters.wait_for_server_floating_ip(
                    self.servers_client, server, floating_ip,
                    wait_for_disassociate=True)

            start += len(volume_types)
            end += len(volume_types)

    def _attach_fixup(self, server, volume):
        """Attach a volume to the server and update the device key with the
        device actually created inside the guest.
        """
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume['id'], 'available')

        list_blks = "lsblk --nodeps --noheadings --output NAME"

        blks_before = set(self.linux_client.exec_command(
            list_blks).strip().splitlines())

        attached_volume = self.nova_volume_attach(server, volume)
        # dev name volume['attachments'][0]['device'][5:] is like
        # /dev/vdb, we need to remove /dev/ -> first 5 chars
        dev_name = attached_volume['attachments'][0]['device'][5:]

        retry = 0
        actual_dev = None
        blks_now = set()
        while retry < 4 and not actual_dev:
            try:
                blks_now = set(self.linux_client.exec_command(
                    list_blks).strip().splitlines())
                for blk_dev in (blks_now - blks_before):
                    serial = self.linux_client.exec_command(
                        f"cat /sys/block/{blk_dev}/serial")
                    if serial == volume['id'][:len(serial)]:
                        actual_dev = blk_dev
                        break
            except exceptions.SSHExecCommandFailed:
                retry += 1
                time.sleep(2 ** retry)

        if not actual_dev and len(blks_now - blks_before):
            LOG.warning("Detected new devices in guest but could not match any"
                        f" of them with the volume {volume['id']}")

        if actual_dev and dev_name != actual_dev:
            LOG.info(
                f"OpenStack mapping {volume['id']} to device {dev_name}" +
                f" is actually {actual_dev} inside the guest")
            dev_name = actual_dev

        return attached_volume, dev_name



# Copyright 2015 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common import cred_client
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


def get_project_by_name(client, project_name):
    projects = client.list_projects({'name': project_name})['projects']
    for project in projects:
        if project['name'] == project_name:
            return project
    raise lib_exc.NotFound('No such project(%s) in %s' % (project_name,
                                                          projects))


def get_tenant_by_name(client, tenant_name):
    tenants = client.list_tenants()['tenants']
    for tenant in tenants:
        if tenant['name'] == tenant_name:
            return tenant
    raise lib_exc.NotFound('No such tenant(%s) in %s' % (tenant_name, tenants))


def get_user_by_username(client, tenant_id, username):
    users = client.list_tenant_users(tenant_id)['users']
    for user in users:
        if user['name'] == username:
            return user
    raise lib_exc.NotFound('No such user(%s) in %s' % (username, users))


def get_user_by_project(users_client, roles_client, project_id, username):
    users = users_client.list_users(**{'name': username})['users']
    users_in_project = roles_client.list_role_assignments(
        **{'scope.project.id': project_id})['role_assignments']
    for user in users:
        if user['name'] == username:
            for u in users_in_project:
                if u['user']['id'] == user['id']:
                    return user
    raise lib_exc.NotFound('No such user(%s) in %s' % (username, users))


def identity_utils(clients):
    """A client that abstracts v2 and v3 identity operations.

    This can be used for creating and tearing down projects in tests. It
    should not be used for testing identity features.

    :param clients: a client manager.
    :return: v2 or v3 of CredsClient
    :rtype: V2CredsClient or V3CredsClient
    """
    if CONF.identity.auth_version == 'v2':
        client = clients.identity_client
        users_client = clients.users_client
        project_client = clients.tenants_client
        roles_client = clients.roles_client
        domains_client = None
    else:
        client = clients.identity_v3_client
        users_client = clients.users_v3_client
        project_client = clients.projects_client
        roles_client = clients.roles_v3_client
        domains_client = clients.domains_client

    try:
        domain = client.auth_provider.credentials.project_domain_name
    except AttributeError:
        domain = CONF.auth.default_credentials_domain_name

    return cred_client.get_creds_client(client, project_client,
                                        users_client,
                                        roles_client,
                                        domains_client,
                                        project_domain_name=domain)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from oslo_log import log as logging
import stevedore

from tempest.lib.common.utils import misc
from tempest.lib.services import clients

LOG = logging.getLogger(__name__)


class TempestPlugin(object, metaclass=abc.ABCMeta):
    """Provide basic hooks for an external plugin

    To provide tempest the necessary information to run the plugin.
    """

    @abc.abstractmethod
    def load_tests(self):
        """Return the information necessary to load the tests in the plugin.

        :return: a tuple with the first value being the test_dir and the second
                 being the top_level
        :rtype: tuple
        """
        return

    @abc.abstractmethod
    def register_opts(self, conf):
        """Add additional configuration options to tempest.

        This method will be run for the plugin during the register_opts()
        function in tempest.config.

        :param ConfigOpts conf: The conf object that can be used to register
            additional options on.

        Example::

            # Config options are defined in a config.py module
            service_option = cfg.BoolOpt(
                "my_service",
                default=True,
                help="Whether or not my service is available")

            # Note: as long as the group is listed in get_opt_lists,
            # it will be possible to access its options in the plugin code
            # via ("-" in the group name are replaces with "_"):
            #     CONF.my_service.<option_name>
            my_service_group = cfg.OptGroup(name="my-service",
                                            title="My service options")

            MyServiceGroup = [<list of options>]
            # (...) More groups and options...

            # Plugin is implemented in a plugin.py module
            from my_plugin import config as my_config

            def register_opts(self, conf):
                conf.register_opt(my_config.service_option,
                                  group='service_available')
                conf.register_group(my_config.my_service_group)
                conf.register_opts(my_config.MyServiceGroup,
                                   my_config.my_service_group)

                conf.register_group(my_config.my_service_feature_group)
                conf.register_opts(my_config.MyServiceFeaturesGroup,
                                   my_config.my_service_feature_group)
        """
        return

    @abc.abstractmethod
    def get_opt_lists(self):
        """Get a list of options for sample config generation

        :return option_list: A list of tuples with the group name and options
                             in that group.
        :rtype: list

        Example::

            # Config options are defined in a config.py module
            service_option = cfg.BoolOpt(
                "my_service", default=True,
                help="Whether or not my service is available")

            my_service_group = cfg.OptGroup(name="my-service",
                                            title="My service options")
            my_service_features_group = cfg.OptGroup(
                name="my-service-features",
                title="My service available features")

            MyServiceGroup = [<list of options>]
            MyServiceFeaturesGroup = [<list of options>]

            # Plugin is implemented in a plugin.py module
            from my_plugin import config as my_config

            def get_opt_lists(self, conf):
                return [
                    (my_service_group.name, MyServiceGroup),
                    (my_service_features_group.name, MyServiceFeaturesGroup)
                ]
        """
        return []

    def get_service_clients(self):
        """Get a list of the service clients for registration

        If the plugin implements service clients for one or more APIs, it
        may return their details by this method for automatic registration
        in any ServiceClients object instantiated by tests.
        The default implementation returns an empty list.

        :returns: Each element of the list represents the service client for an
          API. Each dict must define all parameters required for the invocation
          of `service_clients.ServiceClients.register_service_client_module`.
        :rtype: list of dictionaries

        Example implementation with one service client::

            def get_service_clients(self):
                # Example implementation with one service client
                myservice_config = config.service_client_config('myservice')
                params = {
                    'name': 'myservice',
                    'service_version': 'myservice',
                    'module_path': 'myservice_tempest_tests.services',
                    'client_names': ['API1Client', 'API2Client'],
                }
                params.update(myservice_config)
                return [params]

        Example implementation with two service clients::

            def get_service_clients(self):
                # Example implementation with two service clients
                foo1_config = config.service_client_config('foo')
                params_foo1 = {
                    'name': 'foo_v1',
                    'service_version': 'foo.v1',
                    'module_path': 'bar_tempest_tests.services.foo.v1',
                    'client_names': ['API1Client', 'API2Client'],
                }
                params_foo1.update(foo_config)
                foo2_config = config.service_client_config('foo')
                params_foo2 = {
                    'name': 'foo_v2',
                    'service_version': 'foo.v2',
                    'module_path': 'bar_tempest_tests.services.foo.v2',
                    'client_names': ['API1Client', 'API2Client'],
                }
                params_foo2.update(foo2_config)
                return [params_foo1, params_foo2]
        """
        return []


@misc.singleton
class TempestTestPluginManager(object):
    """Tempest test plugin manager class

    This class is used to manage the lifecycle of external tempest test
    plugins. It provides functions for getting set
    """

    def __init__(self):
        self.ext_plugins = stevedore.ExtensionManager(
            'tempest.test_plugins', invoke_on_load=True,
            propagate_map_exceptions=True,
            on_load_failure_callback=self.failure_hook)

    @staticmethod
    def failure_hook(_, ep, err):
        LOG.error('Could not load %r: %s', ep.name, err)
        raise err

    def get_plugin_load_tests_tuple(self):
        load_tests_dict = {}
        for plug in self.ext_plugins:
            LOG.info('Loading tests from Tempest plugin: %s', plug.name)
            load_tests_dict[plug.name] = plug.obj.load_tests()
        return load_tests_dict

    def register_plugin_opts(self, conf):
        for plug in self.ext_plugins:
            LOG.info('Register additional config options from Tempest '
                     'plugin: %s', plug.name)
            try:
                plug.obj.register_opts(conf)
            except Exception:
                LOG.exception('Plugin %s raised an exception trying to run '
                              'register_opts', plug.name)

    def get_plugin_options_list(self):
        plugin_options = []
        for plug in self.ext_plugins:
            opt_list = plug.obj.get_opt_lists()
            LOG.info('List additional config options registered by '
                     'Tempest plugin: %s', plug.name)

            if opt_list:
                plugin_options.extend(opt_list)
        return plugin_options

    def _register_service_clients(self):
        registry = clients.ClientsRegistry()
        for plug in self.ext_plugins:
            try:
                service_clients = plug.obj.get_service_clients()
                if service_clients:
                    registry.register_service_client(
                        plug.name, service_clients)
            except Exception:
                LOG.exception('Plugin %s raised an exception trying to run '
                              'get_service_clients', plug.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Runs tempest tests

This command is used for running the tempest tests

Test Selection
==============
Tempest run has several options:

* ``--regex/-r``: This is a selection regex like what stestr uses. It will run
  any tests that match on re.match() with the regex
* ``--smoke/-s``: Run all the tests tagged as smoke
* ``--exclude-regex``: It allows to do simple test exclusion via passing a
  rejection/exclude regexp

There are also the ``--exclude-list`` and ``--include-list`` options that
let you pass a filepath to tempest run with the file format being a line
separated regex, with '#' used to signify the start of a comment on a line.
For example::

    # Regex file
    ^regex1 # Match these tests
    .*regex2 # Match those tests

These arguments are just passed into stestr, you can refer to the stestr
selection docs for more details on how these operate:
http://stestr.readthedocs.io/en/latest/MANUAL.html#test-selection

You can also use the ``--list-tests`` option in conjunction with selection
arguments to list which tests will be run.

You can also use the ``--load-list`` option that lets you pass a filepath to
tempest run with the file format being in a non-regex format, similar to the
tests generated by the ``--list-tests`` option. You can specify target tests
by removing unnecessary tests from a list file which is generated from
``--list-tests`` option.

You can also use ``--worker-file`` option that let you pass a filepath to a
worker yaml file, allowing you to manually schedule the tests run.
For example, you can setup a tempest run with
different concurrences to be used with different regexps.
An example of worker file is showed below::

    # YAML Worker file
    - worker:
      # you can have more than one regex per worker
      - tempest.api.*
      - neutron_tempest_tests
    - worker:
      - tempest.scenario.*

This will run test matching with 'tempest.api.*' and 'neutron_tempest_tests'
against worker 1. Run tests matching with 'tempest.scenario.*' under worker 2.

You can mix manual scheduling with the standard scheduling mechanisms by
concurrency field on a worker. For example::

    # YAML Worker file
    - worker:
      # you can have more than one regex per worker
      - tempest.api.*
      - neutron_tempest_tests
      concurrency: 3
    - worker:
      - tempest.scenario.*
      concurrency: 2

This will run tests matching with 'tempest.scenario.*' against 2 workers.

This worker file is passed into stestr. For some more details on how it
operates please refer to the stestr scheduling docs:
https://stestr.readthedocs.io/en/stable/MANUAL.html#test-scheduling

Test Execution
==============
There are several options to control how the tests are executed. By default
tempest will run in parallel with a worker for each CPU present on the machine.
If you want to adjust the number of workers use the ``--concurrency`` option
and if you want to run tests serially use ``--serial/-t``

Running with Workspaces
-----------------------
Tempest run enables you to run your tempest tests from any setup tempest
workspace it relies on you having setup a tempest workspace with either the
``tempest init`` or ``tempest workspace`` commands. Then using the
``--workspace`` CLI option you can specify which one of your workspaces you
want to run tempest from. Using this option you don't have to run Tempest
directly with you current working directory being the workspace, Tempest will
take care of managing everything to be executed from there.

Running from Anywhere
---------------------
Tempest run provides you with an option to execute tempest from anywhere on
your system. You are required to provide a config file in this case with the
``--config-file`` option. When run tempest will create a .stestr
directory and a .stestr.conf file in your current working directory. This way
you can use stestr commands directly to inspect the state of the previous run.

Test Output
===========
By default tempest run's output to STDOUT will be generated using the
subunit-trace output filter. But, if you would prefer a subunit v2 stream be
output to STDOUT use the ``--subunit`` flag

Combining Runs
==============

There are certain situations in which you want to split a single run of tempest
across 2 executions of tempest run. (for example to run part of the tests
serially and others in parallel) To accomplish this but still treat the results
as a single run you can leverage the ``--combine`` option which will append
the current run's results with the previous runs.
"""

import os
import sys

from cliff import command
from oslo_log import log
from oslo_serialization import jsonutils as json
from stestr import commands

from tempest import clients
from tempest.cmd import cleanup_service
from tempest.cmd import init
from tempest.cmd import workspace
from tempest.common import credentials_factory as credentials
from tempest import config

CONF = config.CONF
SAVED_STATE_JSON = "saved_state.json"

LOG = log.getLogger(__name__)


class TempestRun(command.Command):

    def _set_env(self, config_file=None):
        if config_file:
            if os.path.exists(os.path.abspath(config_file)):
                CONF.set_config_path(os.path.abspath(config_file))
            else:
                raise FileNotFoundError(
                    "Config file: %s doesn't exist" % config_file)

        # NOTE(mtreinish): This is needed so that stestr doesn't gobble up any
        # stacktraces on failure.
        if 'TESTR_PDB' in os.environ:
            return
        else:
            os.environ["TESTR_PDB"] = ""
        # NOTE(dims): most of our .stestr.conf try to test for PYTHON
        # environment variable and fall back to "python", under python3
        # if it does not exist. we should set it to the python3 executable
        # to deal with this situation better for now.
        if 'PYTHON' not in os.environ:
            os.environ['PYTHON'] = sys.executable

    def _create_stestr_conf(self):
        top_level_path = os.path.dirname(os.path.dirname(__file__))
        discover_path = os.path.join(top_level_path, 'test_discover')
        file_contents = init.STESTR_CONF % (discover_path, top_level_path)
        with open('.stestr.conf', 'w+') as stestr_conf_file:
            stestr_conf_file.write(file_contents)

    def take_action(self, parsed_args):
        if parsed_args.config_file:
            self._set_env(parsed_args.config_file)
        else:
            self._set_env()
        # Workspace execution mode
        if parsed_args.workspace:
            workspace_mgr = workspace.WorkspaceManager(
                parsed_args.workspace_path)
            path = workspace_mgr.get_workspace(parsed_args.workspace)
            if not path:
                sys.exit(
                    "The %r workspace isn't registered in "
                    "%r. Use 'tempest init' to "
                    "register the workspace." %
                    (parsed_args.workspace, workspace_mgr.path))
            os.chdir(path)
            if not os.path.isfile('.stestr.conf'):
                self._create_stestr_conf()
        # local execution with config file mode
        elif parsed_args.config_file and not os.path.isfile('.stestr.conf'):
            self._create_stestr_conf()
        elif not os.path.isfile('.stestr.conf'):
            print("No .stestr.conf file was found for local execution")
            sys.exit(2)
        if parsed_args.state:
            self._init_state()

        regex = self._build_regex(parsed_args)

        # temporary method for parsing deprecated and new stestr options
        # and showing warning messages in order to make the transition
        # smoother for all tempest consumers
        # TODO(kopecmartin) remove this after stestr>=3.1.0 is used
        # in all supported OpenStack releases
        def parse_dep(old_o, old_v, new_o, new_v):
            ret = ''
            if old_v:
                LOG.warning("'%s' option is deprecated, use '%s' instead "
                            "which is functionally equivalent. Right now "
                            "Tempest still supports this option for "
                            "backward compatibility, however, it will be "
                            "removed soon.",
                            old_o, new_o)
                ret = old_v
            if old_v and new_v:
                # both options are specified
                LOG.warning("'%s' and '%s' are specified at the same time, "
                            "'%s' takes precedence over '%s'",
                            new_o, old_o, new_o, old_o)
            if new_v:
                ret = new_v
            return ret
        ex_regex = parse_dep('--black-regex', parsed_args.black_regex,
                             '--exclude-regex', parsed_args.exclude_regex)
        ex_list = parse_dep('--blacklist-file', parsed_args.blacklist_file,
                            '--exclude-list', parsed_args.exclude_list)
        in_list = parse_dep('--whitelist-file', parsed_args.whitelist_file,
                            '--include-list', parsed_args.include_list)

        return_code = 0
        if parsed_args.list_tests:
            try:
                return_code = commands.list_command(
                    filters=regex, include_list=in_list,
                    exclude_list=ex_list, exclude_regex=ex_regex)
            except TypeError:
                # exclude_list, include_list and exclude_regex are defined only
                # in stestr >= 3.1.0, this except block catches the case when
                # tempest is executed with an older stestr
                return_code = commands.list_command(
                    filters=regex, whitelist_file=in_list,
                    blacklist_file=ex_list, black_regex=ex_regex)

        else:
            serial = not parsed_args.parallel
            params = {
                'filters': regex, 'subunit_out': parsed_args.subunit,
                'serial': serial, 'concurrency': parsed_args.concurrency,
                'worker_path': parsed_args.worker_file,
                'load_list': parsed_args.load_list,
                'combine': parsed_args.combine
            }
            try:
                return_code = commands.run_command(
                    **params, exclude_list=ex_list,
                    include_list=in_list, exclude_regex=ex_regex)
            except TypeError:
                # exclude_list, include_list and exclude_regex are defined only
                # in stestr >= 3.1.0, this except block catches the case when
                # tempest is executed with an older stestr
                return_code = commands.run_command(
                    **params, blacklist_file=ex_list,
                    whitelist_file=in_list, black_regex=ex_regex)
            if parsed_args.slowest:
                commands.slowest_command()
            if return_code > 0:
                sys.exit(return_code)
        return return_code

    def get_description(self):
        return 'Run tempest'

    def _init_state(self):
        print("Initializing saved state.")
        data = {}
        self.global_services = cleanup_service.get_global_cleanup_services()
        self.admin_mgr = clients.Manager(
            credentials.get_configured_admin_credentials())
        admin_mgr = self.admin_mgr
        kwargs = {'data': data,
                  'is_dry_run': False,
                  'saved_state_json': data,
                  'is_preserve': False,
                  'is_save_state': True}
        for service in self.global_services:
            svc = service(admin_mgr, **kwargs)
            svc.run()

        with open(SAVED_STATE_JSON, 'w+') as f:
            f.write(json.dumps(data, sort_keys=True,
                               indent=2, separators=(',', ': ')))

    def get_parser(self, prog_name):
        parser = super(TempestRun, self).get_parser(prog_name)
        parser = self._add_args(parser)
        return parser

    def _add_args(self, parser):
        # workspace args
        parser.add_argument('--workspace', default=None,
                            help='Name of tempest workspace to use for running'
                                 ' tests. You can see a list of workspaces '
                                 'with tempest workspace list')
        parser.add_argument('--workspace-path', default=None,
                            dest='workspace_path',
                            help="The path to the workspace file, the default "
                                 "is ~/.tempest/workspace.yaml")
        # Configuration flags
        parser.add_argument('--config-file', default=None, dest='config_file',
                            help='Configuration file to run tempest with')
        # test selection args
        regex = parser.add_mutually_exclusive_group()
        regex.add_argument('--smoke', '-s', action='store_true',
                           help="Run the smoke tests only")
        regex.add_argument('--regex', '-r', default='',
                           help='A normal stestr selection regex used to '
                                'specify a subset of tests to run')
        parser.add_argument('--black-regex', dest='black_regex',
                            help='DEPRECATED: This option is deprecated and '
                                 'will be removed soon, use --exclude-regex '
                                 'which is functionally equivalent. If this '
                                 'is specified at the same time as '
                                 '--exclude-regex, this flag will be ignored '
                                 'and --exclude-regex will be used')
        parser.add_argument('--exclude-regex', dest='exclude_regex',
                            help='A regex to exclude tests that match it')
        parser.add_argument('--whitelist-file', '--whitelist_file',
                            help='DEPRECATED: This option is deprecated and '
                                 'will be removed soon, use --include-list '
                                 'which is functionally equivalent. If this '
                                 'is specified at the same time as '
                                 '--include-list, this flag will be ignored '
                                 'and --include-list will be used')
        parser.add_argument('--include-list', '--include_list',
                            help="Path to an include file which contains the "
                                 "regex for tests to be included in tempest "
                                 "run, this file contains a separate regex on "
                                 "each newline.")
        parser.add_argument('--blacklist-file', '--blacklist_file',
                            help='DEPRECATED: This option is deprecated and '
                                 'will be removed soon, use --exclude-list '
                                 'which is functionally equivalent. If this '
                                 'is specified at the same time as '
                                 '--exclude-list, this flag will be ignored '
                                 'and --exclude-list will be used')
        parser.add_argument('--exclude-list', '--exclude_list',
                            help='Path to an exclude file which contains the '
                                 'regex for tests to be excluded in tempest '
                                 'run, this file contains a separate regex on '
                                 'each newline.')
        parser.add_argument('--load-list', '--load_list',
                            help='Path to a non-regex whitelist file, '
                                 'this file contains a separate test '
                                 'on each newline. This command '
                                 'supports files created by the tempest '
                                 'run ``--list-tests`` command')
        parser.add_argument('--worker-file', '--worker_file',
                            help='Optional path to a worker file. This file '
                            'contains each worker configuration to be '
                            'used to schedule the tests run')
        # list only args
        parser.add_argument('--list-tests', '-l', action='store_true',
                            help='List tests',
                            default=False)
        # execution args
        parser.add_argument('--concurrency', '-w',
                            type=int, default=0,
                            help="The number of workers to use, defaults to "
                                 "the number of cpus")
        parallel = parser.add_mutually_exclusive_group()
        parallel.add_argument('--parallel', dest='parallel',
                              action='store_true',
                              help='Run tests in parallel (this is the'
                                   ' default)')
        parallel.add_argument('--serial', '-t', dest='parallel',
                              action='store_false',
                              help='Run tests serially')
        parser.add_argument('--save-state', dest='state',
                            action='store_true',
                            help="To save the state of the cloud before "
                                 "running tempest.")
        # output args
        parser.add_argument("--subunit", action='store_true',
                            help='Enable subunit v2 output')
        parser.add_argument("--combine", action='store_true',
                            help='Combine the output of this run with the '
                                 "previous run's as a combined stream in the "
                                 "stestr repository after it finish")
        parser.add_argument('--slowest', action='store_true',
                            help='Show the longest running tests in the '
                                 'stestr repository after it finishes')

        parser.set_defaults(parallel=True)
        return parser

    def _build_regex(self, parsed_args):
        regex = None
        if parsed_args.smoke:
            regex = ['smoke']
        elif parsed_args.regex:
            regex = parsed_args.regex.split()
        return regex



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import unittest

from tempest.test_discover import plugins


def load_tests(loader, tests, pattern):
    ext_plugins = plugins.TempestTestPluginManager()

    suite = unittest.TestSuite()
    base_path = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
    base_path = os.path.split(base_path)[0]
    # Load local tempest tests
    for test_dir in ['api', 'scenario', 'serial_tests']:
        full_test_dir = os.path.join(base_path, 'tempest', test_dir)
        if not pattern:
            suite.addTests(loader.discover(full_test_dir,
                                           top_level_dir=base_path))
        else:
            suite.addTests(loader.discover(full_test_dir, pattern=pattern,
                                           top_level_dir=base_path))

    plugin_load_tests = ext_plugins.get_plugin_load_tests_tuple()
    if not plugin_load_tests:
        return suite

    # Load any installed plugin tests
    for plugin in plugin_load_tests:
        test_dir, top_path = plugin_load_tests[plugin]
        if not pattern:
            suite.addTests(loader.discover(test_dir, top_level_dir=top_path))
        else:
            suite.addTests(loader.discover(test_dir, pattern=pattern,
                                           top_level_dir=top_path))
    return suite



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

create_group = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'group': {
                'type': 'object',
                'properties': {
                    'id': {'type': 'string', 'format': 'uuid'},
                    'name': {'type': 'string'},
                },
                'additionalProperties': False,
                'required': ['id', 'name']
            }
        },
        'additionalProperties': False,
        'required': ['group']
    }
}

delete_group = {'status_code': [202]}

show_group = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'group': {
                'type': 'object',
                'properties': {
                    'status': {'type': 'string'},
                    'description': {'type': ['string', 'null']},
                    'availability_zone': {'type': 'string'},
                    'created_at': parameter_types.date_time,
                    'group_type': {'type': 'string', 'format': 'uuid'},
                    'group_snapshot_id': {'type': ['string', 'null']},
                    'source_group_id': {'type': ['string', 'null']},
                    'volume_types': {
                        'type': 'array',
                        'items': {'type': 'string', 'format': 'uuid'}
                    },
                    'id': {'type': 'string', 'format': 'uuid'},
                    'name': {'type': 'string'},
                    # TODO(zhufl): volumes is added in 3.25, we should move it
                    # to the 3.25 schema file when microversion is supported
                    # in volume interfaces
                    'volumes': {
                        'type': 'array',
                        'items': {'type': 'string', 'format': 'uuid'}
                    },
                    # TODO(zhufl): replication_status is added in 3.38, we
                    # should move it to the 3.38 schema file when microversion
                    # is supported in volume interfaces
                    'replication_status': {'type': ['string', 'null']}
                },
                'additionalProperties': False,
                'required': ['status', 'description', 'created_at',
                             'group_type', 'volume_types', 'id', 'name']
            }
        },
        'additionalProperties': False,
        'required': ['group']
    }
}

list_groups_no_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'groups': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': 'string', 'format': 'uuid'},
                        'name': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['id', 'name'],
                }
            }
        },
        'additionalProperties': False,
        'required': ['groups'],
    }
}

list_groups_with_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'groups': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'status': {'type': 'string'},
                        'description': {'type': ['string', 'null']},
                        'availability_zone': {'type': 'string'},
                        'created_at': parameter_types.date_time,
                        'group_type': {'type': 'string', 'format': 'uuid'},
                        'group_snapshot_id': {'type': ['string', 'null']},
                        'source_group_id': {'type': ['string', 'null']},
                        'volume_types': {
                            'type': 'array',
                            'items': {'type': 'string', 'format': 'uuid'}
                        },
                        'id': {'type': 'string', 'format': 'uuid'},
                        'name': {'type': 'string'},
                        # TODO(zhufl): volumes is added in 3.25, we should
                        # move it to the 3.25 schema file when microversion
                        # is supported in volume interfaces
                        'volumes': {
                            'type': 'array',
                            'items': {'type': 'string', 'format': 'uuid'}
                        },
                        # TODO(zhufl): replication_status is added in 3.38, we
                        # should move it to the 3.38 schema file when
                        # microversion is supported in volume interfaces
                        'replication_status': {'type': ['string', 'null']}
                    },
                    'additionalProperties': False,
                    'required': ['status', 'description', 'created_at',
                                 'group_type', 'volume_types', 'id', 'name']
                }
            }
        },
        'additionalProperties': False,
        'required': ['groups'],
    }
}

create_group_from_source = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'group': {
                'type': 'object',
                'properties': {
                    'id': {'type': 'string', 'format': 'uuid'},
                    'name': {'type': 'string'},
                },
                'additionalProperties': False,
                'required': ['id', 'name']
            }
        },
        'additionalProperties': False,
        'required': ['group']
    }
}
update_group = {'status_code': [202]}
reset_group_status = {'status_code': [202]}



# Copyright 2013 Hewlett-Packard Development Company, L.P.
# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ServerRescueNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of server rescue"""

    @classmethod
    def skip_checks(cls):
        super(ServerRescueNegativeTestJSON, cls).skip_checks()
        if not CONF.compute_feature_enabled.rescue:
            msg = "Server rescue not available."
            raise cls.skipException(msg)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True, router=True,
                                  dhcp=True)
        super(ServerRescueNegativeTestJSON, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(ServerRescueNegativeTestJSON, cls).resource_setup()
        cls.password = data_utils.rand_password()
        rescue_password = data_utils.rand_password()
        # Server for negative tests
        server = cls.create_test_server(adminPass=cls.password,
                                        wait_until='BUILD')
        resc_server = cls.create_test_server(adminPass=rescue_password,
                                             wait_until='ACTIVE')
        cls.server_id = server['id']
        cls.rescue_id = resc_server['id']

        cls.servers_client.rescue_server(
            cls.rescue_id, adminPass=rescue_password)
        waiters.wait_for_server_status(cls.servers_client,
                                       cls.rescue_id, 'RESCUE')
        waiters.wait_for_server_status(cls.servers_client,
                                       cls.server_id, 'ACTIVE')

    def _unrescue(self, server_id):
        self.servers_client.unrescue_server(server_id)
        waiters.wait_for_server_status(self.servers_client,
                                       server_id, 'ACTIVE')

    def _unpause(self, server_id):
        self.servers_client.unpause_server(server_id)
        waiters.wait_for_server_status(self.servers_client,
                                       server_id, 'ACTIVE')

    @decorators.idempotent_id('cc3a883f-43c0-4fb6-a9bb-5579d64984ed')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @decorators.attr(type=['negative'])
    def test_rescue_paused_instance(self):
        """Test rescuing a paused server should fail"""
        self.servers_client.pause_server(self.server_id)
        self.addCleanup(self._unpause, self.server_id)
        waiters.wait_for_server_status(self.servers_client,
                                       self.server_id, 'PAUSED')
        self.assertRaises(lib_exc.Conflict,
                          self.servers_client.rescue_server,
                          self.server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('db22b618-f157-4566-a317-1b6d467a8094')
    def test_rescued_vm_reboot(self):
        """Test rebooing a rescued server should fail"""
        self.assertRaises(lib_exc.Conflict, self.servers_client.reboot_server,
                          self.rescue_id, type='HARD')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6dfc0a55-3a77-4564-a144-1587b7971dde')
    def test_rescue_non_existent_server(self):
        """Test rescuing a non-existing server should fail"""
        non_existent_server = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.servers_client.rescue_server,
                          non_existent_server)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('70cdb8a1-89f8-437d-9448-8844fd82bf46')
    def test_rescued_vm_rebuild(self):
        """Test rebuilding a rescued server should fail"""
        self.assertRaises(lib_exc.Conflict,
                          self.servers_client.rebuild_server,
                          self.rescue_id,
                          self.image_ref_alt)

    @decorators.idempotent_id('d0ccac79-0091-4cf4-a1ce-26162d0cc55f')
    @utils.services('volume')
    @decorators.attr(type=['negative'])
    def test_rescued_vm_attach_volume(self):
        """Test attaching volume to a rescued server should fail"""
        volume = self.create_volume()

        # Rescue the server
        self.servers_client.rescue_server(self.server_id,
                                          adminPass=self.password)
        waiters.wait_for_server_status(self.servers_client,
                                       self.server_id, 'RESCUE')
        self.addCleanup(self._unrescue, self.server_id)

        # Attach the volume to the server
        self.assertRaises(lib_exc.Conflict,
                          self.servers_client.attach_volume,
                          self.server_id,
                          volumeId=volume['id'])

    @decorators.idempotent_id('f56e465b-fe10-48bf-b75d-646cda3a8bc9')
    @utils.services('volume')
    @decorators.attr(type=['negative'])
    def test_rescued_vm_detach_volume(self):
        """Test detaching volume from a rescued server should fail"""
        volume = self.create_volume()
        # This test just check detach fail and does not
        # perform the detach operation but in cleanup from
        # self.attach_volume() it will try to detach the server
        # after unrescue the server. Due to that we need to make
        # server SSHable before it try to detach, more details are
        # in bug#1960346
        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        server = self.create_test_server(
            adminPass=self.password,
            wait_until="SSHABLE",
            validatable=True,
            validation_resources=validation_resources)
        # Attach the volume to the server
        self.attach_volume(server, volume)

        # Rescue the server
        self.servers_client.rescue_server(server['id'],
                                          adminPass=self.password)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'RESCUE')
        # NOTE(gmann) In next addCleanup, server unrescue is called before the
        # detach volume is called in cleanup (added by self.attach_volume()
        # method) so to make sure server is ready before detach operation, we
        # need to perform ssh on it, more details are in bug#1960346.
        if CONF.validation.run_validation:
            tenant_network = self.get_tenant_network()
            self.addCleanup(compute.wait_for_ssh_or_ping,
                            server, self.os_primary, tenant_network,
                            True, validation_resources, "SSHABLE", True)
        # addCleanup is a LIFO queue
        self.addCleanup(self._unrescue, server['id'])

        # Detach the volume from the server expecting failure
        self.assertRaises(lib_exc.Conflict,
                          self.servers_client.detach_volume,
                          server['id'],
                          volume['id'])



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


LOG = logging.getLogger(__name__)


class ExtensionsTestJSON(base.BaseVolumeTest):
    """Test volume extensions"""

    @decorators.idempotent_id('94607eb0-43a5-47ca-82aa-736b41bd2e2c')
    def test_list_extensions(self):
        """Test listing volume extensions"""
        extensions = (self.volumes_extension_client.list_extensions()
                      ['extensions'])
        if not CONF.volume_feature_enabled.api_extensions:
            raise self.skipException('There are not any extensions configured')
        extension_list = [extension.get('alias') for extension in extensions]
        LOG.debug("Cinder extensions: %s", ','.join(extension_list))
        ext = CONF.volume_feature_enabled.api_extensions[0]
        if ext == 'all':
            self.assertIn('Hosts', map(lambda x: x['name'], extensions))
        elif ext:
            self.assertIn(ext, map(lambda x: x['alias'], extensions))
        else:
            raise self.skipException('There are not any extensions configured')
